@@ -4,8 +4,6 @@
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
-import inspect as _inspect
-import sys
 
 from .sql import (
     alias,
@@ -23,6 +21,7 @@ from .sql import (
     except_all,
     exists,
     extract,
+    false,
     func,
     insert,
     intersect,
@@ -40,6 +39,7 @@ from .sql import (
     select,
     subquery,
     text,
+    true,
     tuple_,
     type_coerce,
     union,
@@ -97,7 +97,6 @@ from .schema import (
     Column,
     ColumnDefault,
     Constraint,
-    DDL,
     DefaultClause,
     FetchedValue,
     ForeignKey,
@@ -110,19 +109,25 @@ from .schema import (
     Table,
     ThreadLocalMetaData,
     UniqueConstraint,
-    )
+    DDL,
+)
+
 
 from .inspection import inspect
-
 from .engine import create_engine, engine_from_config
 
+__version__ = '0.9.1'
 
-__all__ = sorted(name for name, obj in locals().items()
+def __go(lcls):
+    global __all__
+
+    from . import events
+    from . import util as _sa_util
+
+    import inspect as _inspect
+
+    __all__ = sorted(name for name, obj in lcls.items()
                  if not (name.startswith('_') or _inspect.ismodule(obj)))
 
-__version__ = '0.8.7'
-
-del _inspect, sys
-
-from . import util as _sa_util
-_sa_util.importlater.resolve_all()
+    _sa_util.dependencies.resolve_all("sqlalchemy")
+__go(locals())
\ No newline at end of file
