@@ -1,11 +1,12 @@
 # connectors/zxJDBC.py
-# Copyright (C) 2005-2012 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 import sys
-from sqlalchemy.connectors import Connector
+from . import Connector
+
 
 class ZxJDBCConnector(Connector):
     driver = 'zxjdbc'
@@ -33,7 +34,7 @@ class ZxJDBCConnector(Connector):
     def _create_jdbc_url(self, url):
         """Create a JDBC url from a :class:`~sqlalchemy.engine.url.URL`"""
         return 'jdbc:%s://%s%s/%s' % (self.jdbc_db_name, url.host,
-                                      url.port is not None 
+                                      url.port is not None
                                         and ':%s' % url.port or '',
                                       url.database)
 
@@ -41,12 +42,12 @@ class ZxJDBCConnector(Connector):
         opts = self._driver_kwargs()
         opts.update(url.query)
         return [
-                [self._create_jdbc_url(url), 
-                url.username, url.password, 
+                [self._create_jdbc_url(url),
+                url.username, url.password,
                 self.jdbc_driver_name],
                 opts]
 
-    def is_disconnect(self, e):
+    def is_disconnect(self, e, connection, cursor):
         if not isinstance(e, self.dbapi.ProgrammingError):
             return False
         e = str(e)
