@@ -224,7 +224,7 @@ class MSExecutionContext_pyodbc(MSExecutionContext):
                     # without closing it (FreeTDS particularly)
                     row = self.cursor.fetchall()[0]
                     break
-                except self.dialect.dbapi.Error, e:
+                except self.dialect.dbapi.Error as e:
                     # no way around this - nextset() consumes the previous set
                     # so we need to just keep flipping
                     self.cursor.nextset()
@@ -249,9 +249,8 @@ class MSDialect_pyodbc(PyODBCConnector, MSDialect):
     )
 
     def __init__(self, description_encoding=None, **params):
-        if 'description_encoding' in params:
-            self.description_encoding = params.pop('description_encoding')
         super(MSDialect_pyodbc, self).__init__(**params)
+        self.description_encoding = description_encoding
         self.use_scope_identity = self.use_scope_identity and \
                         self.dbapi and \
                         hasattr(self.dbapi.Cursor, 'nextset')
