@@ -475,12 +475,11 @@ class SybaseDialect(default.DefaultDialect):
               AND o.type in ('U', 'V')
         """)
 
-        # Py2K
-        if isinstance(schema, unicode):
-            schema = schema.encode("ascii")
-        if isinstance(table_name, unicode):
-            table_name = table_name.encode("ascii")
-        # end Py2K
+        if util.py2k:
+            if isinstance(schema, unicode):
+                schema = schema.encode("ascii")
+            if isinstance(table_name, unicode):
+                table_name = table_name.encode("ascii")
         result = connection.execute(TABLEID_SQL,
                                     schema_name=schema,
                                     table_name=table_name)
@@ -759,10 +758,10 @@ class SybaseDialect(default.DefaultDialect):
             AND o.type = 'U'
         """)
 
-        # Py2K
-        if isinstance(schema, unicode):
-            schema = schema.encode("ascii")
-        # end Py2K
+        if util.py2k:
+            if isinstance(schema, unicode):
+                schema = schema.encode("ascii")
+
         tables = connection.execute(TABLE_SQL, schema_name=schema)
 
         return [t["name"] for t in tables]
@@ -779,10 +778,10 @@ class SybaseDialect(default.DefaultDialect):
             AND o.type = 'V'
         """)
 
-        # Py2K
-        if isinstance(view_name, unicode):
-            view_name = view_name.encode("ascii")
-        # end Py2K
+        if util.py2k:
+            if isinstance(view_name, unicode):
+                view_name = view_name.encode("ascii")
+
         view = connection.execute(VIEW_DEF_SQL, view_name=view_name)
 
         return view.scalar()
@@ -799,10 +798,9 @@ class SybaseDialect(default.DefaultDialect):
             AND o.type = 'V'
         """)
 
-        # Py2K
-        if isinstance(schema, unicode):
-            schema = schema.encode("ascii")
-        # end Py2K
+        if util.py2k:
+            if isinstance(schema, unicode):
+                schema = schema.encode("ascii")
         views = connection.execute(VIEW_SQL, schema_name=schema)
 
         return [v["name"] for v in views]
