@@ -49,6 +49,7 @@ tracks all parents which reference it.  Below, we illustrate the a simple
 version of the :class:`.MutableDict` dictionary object, which applies
 the :class:`.Mutable` mixin to a plain Python dictionary::
 
+    import collections
     from sqlalchemy.ext.mutable import Mutable
 
     class MutableDict(Mutable, dict):
@@ -177,7 +178,7 @@ callbacks. In our case, this is a good thing, since if this dictionary were
 picklable, it could lead to an excessively large pickle size for our value
 objects that are pickled by themselves outside of the context of the parent.
 The developer responsibility here is only to provide a ``__getstate__`` method
-that excludes the :meth:`~.MutableBase._parents` collection from the pickle
+that excludes the :meth:`~MutableBase._parents` collection from the pickle
 stream::
 
     class MyMutableType(Mutable):
@@ -327,7 +328,7 @@ Supporting Pickling
 
 As is the case with :class:`.Mutable`, the :class:`.MutableComposite` helper
 class uses a ``weakref.WeakKeyDictionary`` available via the
-:meth:`.MutableBase._parents` attribute which isn't picklable. If we need to
+:meth:`MutableBase._parents` attribute which isn't picklable. If we need to
 pickle instances of ``Point`` or its owning class ``Vertex``, we at least need
 to define a ``__getstate__`` that doesn't include the ``_parents`` dictionary.
 Below we define both a ``__getstate__`` and a ``__setstate__`` that package up
@@ -344,7 +345,7 @@ the minimal form of our ``Point`` class::
 
 As with :class:`.Mutable`, the :class:`.MutableComposite` augments the
 pickling process of the parent's object-relational state so that the
-:meth:`.MutableBase._parents` collection is restored to all ``Point`` objects.
+:meth:`MutableBase._parents` collection is restored to all ``Point`` objects.
 
 """
 from ..orm.attributes import flag_modified
@@ -437,9 +438,6 @@ class MutableBase(object):
             outgoing.
 
             """
-            if value is oldvalue:
-                return value
-
             if not isinstance(value, cls):
                 value = cls.coerce(key, value)
             if value is not None:
@@ -540,7 +538,7 @@ class Mutable(MutableBase):
 
         To associate a particular mutable type with all occurrences of a
         particular type, use the :meth:`.Mutable.associate_with` classmethod
-        of the particular :meth:`.Mutable` subclass to establish a global
+        of the particular :class:`.Mutable` subclass to establish a global
         association.
 
         .. warning::
@@ -586,14 +584,14 @@ class MutableComposite(MutableBase):
                 setattr(parent, attr_name, value)
 
 def _setup_composite_listener():
-    import types
     def _listen_for_type(mapper, class_):
         for prop in mapper.iterate_properties:
-            if (hasattr(prop, 'composite_class') and (type(prop.composite_class) in (types.ClassType, types.TypeType)) and
-                issubclass(prop.composite_class, MutableComposite)):
+            if (hasattr(prop, 'composite_class') and
+                isinstance(prop.composite_class, type) and
+                 issubclass(prop.composite_class, MutableComposite)):
                 prop.composite_class._listen_on_attribute(
                     getattr(class_, prop.key), False, class_)
-    if not Mapper.dispatch.mapper_configured._contains(Mapper, _listen_for_type):
+    if not event.contains(Mapper, "mapper_configured", _listen_for_type):
         event.listen(Mapper, 'mapper_configured', _listen_for_type)
 _setup_composite_listener()
 
@@ -610,11 +608,6 @@ class MutableDict(Mutable, dict):
         dict.__setitem__(self, key, value)
         self.changed()
 
-    def setdefault(self, key, value):
-        result = dict.setdefault(self, key, value)
-        self.changed()
-        return result
-
     def __delitem__(self, key):
         """Detect dictionary del events and emit change events."""
         dict.__delitem__(self, key)
