@@ -9,16 +9,19 @@
 
 """Defines operators used in SQL expressions."""
 
+from .. import util
+
+
 from operator import (
     and_, or_, inv, add, mul, sub, mod, truediv, lt, le, ne, gt, ge, eq, neg,
     getitem, lshift, rshift
     )
 
-# Py2K
-from operator import (div,)
-# end Py2K
+if util.py2k:
+    from operator import div
+else:
+    div = truediv
 
-from ..util import symbol
 
 
 class Operators(object):
@@ -503,17 +506,11 @@ class ColumnOperators(Operators):
         return self.operate(contains_op, other, **kwargs)
 
     def match(self, other, **kwargs):
-        """Implements a database-specific 'match' operator.
+        """Implements the 'match' operator.
 
-        :meth:`~.ColumnOperators.match` attempts to resolve to
-        a MATCH-like function or operator provided by the backend.
-        Examples include:
-
-        * Postgresql - renders ``x @@ to_tsquery(y)``
-        * MySQL - renders ``MATCH (x) AGAINST (y IN BOOLEAN MODE)``
-        * Oracle - renders ``CONTAINS(x, y)``
-        * other backends may provide special implementations;
-          some backends such as SQLite have no support.
+        In a column context, this produces a MATCH clause, i.e.
+        ``MATCH '<other>'``.  The allowed contents of ``other``
+        are database backend specific.
 
         """
         return self.operate(match_op, other, **kwargs)
@@ -660,6 +657,12 @@ def exists():
     raise NotImplementedError()
 
 
+def istrue(a):
+    raise NotImplementedError()
+
+def isfalse(a):
+    raise NotImplementedError()
+
 def is_(a, b):
     return a.is_(b)
 
@@ -785,17 +788,16 @@ parenthesize (a op b).
 
 """
 
-_smallest = symbol('_smallest', canonical=-100)
-_largest = symbol('_largest', canonical=100)
+_asbool = util.symbol('_asbool', canonical=-10)
+_smallest = util.symbol('_smallest', canonical=-100)
+_largest = util.symbol('_largest', canonical=100)
 
 _PRECEDENCE = {
     from_: 15,
     getitem: 15,
     mul: 8,
     truediv: 8,
-    # Py2K
     div: 8,
-    # end Py2K
     mod: 8,
     neg: 8,
     add: 7,
@@ -824,12 +826,19 @@ _PRECEDENCE = {
     between_op: 5,
     distinct_op: 5,
     inv: 5,
+    istrue: 5,
+    isfalse: 5,
     and_: 3,
     or_: 2,
     comma_op: -1,
-    collate: 7,
+
+    desc_op: 3,
+    asc_op: 3,
+    collate: 4,
+
     as_: -1,
     exists: 0,
+    _asbool: -10,
     _smallest: _smallest,
     _largest: _largest
 }
