@@ -1,3 +1,9 @@
+# testing/engines.py
+# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
+#
+# This module is part of SQLAlchemy and is released under
+# the MIT License: http://www.opensource.org/licenses/mit-license.php
+
 from __future__ import absolute_import
 
 import types
@@ -8,7 +14,7 @@ from .util import decorator
 from .. import event, pool
 import re
 import warnings
-
+from .. import util
 
 class ConnectionKiller(object):
 
@@ -31,25 +37,24 @@ class ConnectionKiller(object):
             fn()
         except (SystemExit, KeyboardInterrupt):
             raise
-        except Exception, e:
+        except Exception as e:
             warnings.warn(
                     "testing_reaper couldn't "
                     "rollback/close connection: %s" % e)
 
     def rollback_all(self):
-        for rec in self.proxy_refs.keys():
+        for rec in list(self.proxy_refs):
             if rec is not None and rec.is_valid:
                 self._safe(rec.rollback)
 
     def close_all(self):
-        for rec in self.proxy_refs.keys():
+        for rec in list(self.proxy_refs):
             if rec is not None:
                 self._safe(rec._close)
 
     def _after_test_ctx(self):
-        pass
         # this can cause a deadlock with pg8000 - pg8000 acquires
-        # prepared statement lock inside of rollback() - if async gc
+        # prepared statment lock inside of rollback() - if async gc
         # is collecting in finalize_fairy, deadlock.
         # not sure if this should be if pypy/jython only.
         # note that firebird/fdb definitely needs this though
@@ -67,7 +72,7 @@ class ConnectionKiller(object):
 
         self.conns = set()
 
-        for rec in self.testing_engines.keys():
+        for rec in list(self.testing_engines):
             if rec is not config.db:
                 rec.dispose()
 
@@ -76,9 +81,9 @@ class ConnectionKiller(object):
         for conn, rec in self.conns:
             self._safe(conn.close)
             rec.connection = None
-            
+
         self.conns = set()
-        for rec in self.testing_engines.keys():
+        for rec in list(self.testing_engines):
             rec.dispose()
 
     def assert_all_closed(self):
@@ -163,7 +168,7 @@ class ReconnectFixture(object):
             fn()
         except (SystemExit, KeyboardInterrupt):
             raise
-        except Exception, e:
+        except Exception as e:
             warnings.warn(
                     "ReconnectFixture couldn't "
                     "close connection: %s" % e)
@@ -356,23 +361,22 @@ class ReplayableSession(object):
     Callable = object()
     NoAttribute = object()
 
-    # Py3K
-    #Natives = set([getattr(types, t)
-    #               for t in dir(types) if not t.startswith('_')]). \
-    #               union([type(t) if not isinstance(t, type)
-    #                        else t for t in __builtins__.values()]).\
-    #               difference([getattr(types, t)
-    #                        for t in ('FunctionType', 'BuiltinFunctionType',
-    #                                  'MethodType', 'BuiltinMethodType',
-    #                                  'LambdaType', )])
-    # Py2K
-    Natives = set([getattr(types, t)
-                   for t in dir(types) if not t.startswith('_')]). \
+    if util.py2k:
+        Natives = set([getattr(types, t)
+                   for t in dir(types) if not t.startswith('_')]).\
                    difference([getattr(types, t)
                            for t in ('FunctionType', 'BuiltinFunctionType',
                                      'MethodType', 'BuiltinMethodType',
                                      'LambdaType', 'UnboundMethodType',)])
-    # end Py2K
+    else:
+        Natives = set([getattr(types, t)
+                       for t in dir(types) if not t.startswith('_')]).\
+                       union([type(t) if not isinstance(t, type)
+                                else t for t in __builtins__.values()]).\
+                       difference([getattr(types, t)
+                                for t in ('FunctionType', 'BuiltinFunctionType',
+                                          'MethodType', 'BuiltinMethodType',
+                                          'LambdaType', )])
 
     def __init__(self):
         self.buffer = deque()
