@@ -29,7 +29,7 @@ class CompileTest(fixtures.TestBase, AssertsExecutionResults):
             for c in t.c:
                 c.type._type_affinity
         from sqlalchemy import types
-        for t in types._type_map.values():
+        for t in list(types._type_map.values()):
             t._type_affinity
 
         cls.dialect = default.DefaultDialect()
