@@ -1,10 +1,11 @@
 # coding: utf-8
-from __future__ import with_statement
+
 from sqlalchemy.testing.assertions import eq_, assert_raises, \
                 assert_raises_message, is_, AssertsExecutionResults, \
                 AssertsCompiledSQL, ComparesTables
 from sqlalchemy.testing import engines, fixtures
 from sqlalchemy import testing
+from sqlalchemy import inspect
 from sqlalchemy import Table, Column, select, MetaData, text, Integer, \
             String, Sequence, ForeignKey, join, Numeric, \
             PrimaryKeyConstraint, DateTime, tuple_, Float, BigInteger, \
@@ -19,7 +20,7 @@ class DomainReflectionTest(fixtures.TestBase, AssertsExecutionResults):
 
     """Test PostgreSQL domains"""
 
-    __only_on__ = 'postgresql > 8.3'
+    __only_on__ = 'postgresql'
 
     @classmethod
     def setup_class(cls):
@@ -32,7 +33,7 @@ class DomainReflectionTest(fixtures.TestBase, AssertsExecutionResults):
             :
             try:
                 con.execute(ddl)
-            except exc.DBAPIError, e:
+            except exc.DBAPIError as e:
                 if not 'already exists' in str(e):
                     raise e
         con.execute('CREATE TABLE testtable (question integer, answer '
@@ -127,8 +128,8 @@ class DomainReflectionTest(fixtures.TestBase, AssertsExecutionResults):
 class ReflectionTest(fixtures.TestBase):
     __only_on__ = 'postgresql'
 
-    @testing.fails_if("postgresql < 8.4",
-                    "Better int2vector functions not available")
+    @testing.fails_if(('postgresql', '<', (8, 4)),
+            "newer query is bypassed due to unsupported SQL functions")
     @testing.provide_metadata
     def test_reflected_primary_key_order(self):
         meta1 = self.metadata
@@ -169,8 +170,6 @@ class ReflectionTest(fixtures.TestBase):
         eq_(
             t.c.x.server_default.arg.text, "'%s'::character varying" % ("abcd" * 40)
         )
-
-    @testing.fails_if("postgresql < 8.1", "schema name leaks in, not sure")
     @testing.provide_metadata
     def test_renamed_sequence_reflection(self):
         metadata = self.metadata
@@ -429,6 +428,70 @@ class ReflectionTest(fixtures.TestBase):
         eq_(ind, [{'unique': False, 'column_names': ['y'], 'name': 'idx1'}])
         conn.close()
 
+    @testing.provide_metadata
+    def test_foreign_key_option_inspection(self):
+        metadata = self.metadata
+        Table('person', metadata,
+            Column('id', String(length=32), nullable=False, primary_key=True),
+            Column('company_id', ForeignKey('company.id',
+                name='person_company_id_fkey',
+                match='FULL', onupdate='RESTRICT', ondelete='RESTRICT',
+                deferrable=True, initially='DEFERRED'
+                )
+            )
+        )
+        Table('company', metadata,
+            Column('id', String(length=32), nullable=False, primary_key=True),
+            Column('name', String(length=255)),
+            Column('industry_id', ForeignKey('industry.id',
+                name='company_industry_id_fkey',
+                onupdate='CASCADE', ondelete='CASCADE',
+                deferrable=False,  # PG default
+                initially='IMMEDIATE'  # PG default
+                )
+            )
+        )
+        Table('industry', metadata,
+            Column('id', Integer(), nullable=False, primary_key=True),
+            Column('name', String(length=255))
+        )
+        fk_ref = {
+            'person_company_id_fkey': {
+                'name': 'person_company_id_fkey',
+                'constrained_columns': ['company_id'],
+                'referred_columns': ['id'],
+                'referred_table': 'company',
+                'referred_schema': None,
+                'options': {
+                    'onupdate': 'RESTRICT',
+                    'deferrable': True,
+                    'ondelete': 'RESTRICT',
+                    'initially': 'DEFERRED',
+                    'match': 'FULL'
+                    }
+                },
+            'company_industry_id_fkey': {
+                'name': 'company_industry_id_fkey',
+                'constrained_columns': ['industry_id'],
+                'referred_columns': ['id'],
+                'referred_table': 'industry',
+                'referred_schema': None,
+                'options': {
+                    'onupdate': 'CASCADE',
+                    'deferrable': None,
+                    'ondelete': 'CASCADE',
+                    'initially': None,
+                    'match': None
+                }
+            }
+        }
+        metadata.create_all()
+        inspector = inspect(testing.db)
+        fks = inspector.get_foreign_keys('person') + \
+            inspector.get_foreign_keys('company')
+        for fk in fks:
+            eq_(fk, fk_ref[fk['name']])
+
 class CustomTypeReflectionTest(fixtures.TestBase):
 
     class CustomType(object):
