@@ -61,13 +61,6 @@ class DefaultRequirements(SuiteRequirements):
                     'target backend does not support ON UPDATE CASCADE'
                 )
 
-    @property
-    def non_updating_cascade(self):
-        """target database must *not* support ON UPDATE..CASCADE behavior in
-        foreign keys."""
-
-        return fails_on_everything_except('sqlite', 'oracle', '+zxjdbc') + skip_if('mssql')
-
     @property
     def deferrable_fks(self):
         """target database must support deferrable fks"""
@@ -92,8 +85,6 @@ class DefaultRequirements(SuiteRequirements):
             no_support('oracle', 'not supported by database'),
             no_support('mssql', 'not supported by database'),
             no_support('sybase', 'not supported by database'),
-            no_support('maxdb', 'FIXME: verify not supported by database'),
-            no_support('informix', 'not supported by database'),
         ])
 
     @property
@@ -245,7 +236,6 @@ class DefaultRequirements(SuiteRequirements):
                     "sqlite",
                     "sybase",
                     ("mysql", "<", (5, 0, 3)),
-                    ("informix", "<", (11, 55, "xC3"))
                     ], "savepoints not supported")
 
 
@@ -302,14 +292,14 @@ class DefaultRequirements(SuiteRequirements):
         """Target database must support INTERSECT or equivalent."""
 
         return fails_if([
-                "firebird", "mysql", "sybase", "informix"
+                "firebird", "mysql", "sybase",
             ], 'no support for INTERSECT')
 
     @property
     def except_(self):
         """Target database must support EXCEPT or equivalent (i.e. MINUS)."""
         return fails_if([
-                "firebird", "mysql", "sybase", "informix"
+                "firebird", "mysql", "sybase",
             ], 'no support for EXCEPT')
 
     @property
@@ -332,7 +322,6 @@ class DefaultRequirements(SuiteRequirements):
 
         return skip_if([
             no_support('firebird', 'no SA implementation'),
-            no_support('maxdb', 'two-phase xact not supported by database'),
             no_support('mssql', 'two-phase xact not supported by drivers'),
             no_support('oracle', 'two-phase xact not implemented in SQLA/oracle'),
             no_support('drizzle', 'two-phase xact not supported by database'),
@@ -385,7 +374,6 @@ class DefaultRequirements(SuiteRequirements):
         """Target driver must support some degree of non-ascii symbol names."""
         # TODO: expand to exclude MySQLdb versions w/ broken unicode
         return skip_if([
-            no_support('maxdb', 'database support flakey'),
             no_support('oracle', 'FIXME: no support in database?'),
             no_support('sybase', 'FIXME: guessing, needs confirmation'),
             no_support('mssql+pymssql', 'no FreeTDS support'),
@@ -434,7 +422,7 @@ class DefaultRequirements(SuiteRequirements):
 
     @property
     def sane_multi_rowcount(self):
-        return fails_if(
+        return skip_if(
                     lambda: not self.db.dialect.supports_sane_multi_rowcount,
                     "driver doesn't support 'sane' multi row count"
                 )
@@ -451,6 +439,15 @@ class DefaultRequirements(SuiteRequirements):
         return fails_on_everything_except('postgresql', 'oracle', 'mssql',
                     'sybase')
 
+    @property
+    def datetime_literals(self):
+        """target dialect supports rendering of a date, time, or datetime as a
+        literal string, e.g. via the TypeEngine.literal_processor() method.
+
+        """
+
+        return fails_on_everything_except("sqlite")
+
     @property
     def datetime(self):
         """target dialect supports representation of Python
@@ -606,35 +603,23 @@ class DefaultRequirements(SuiteRequirements):
                 "Python version 3.xx is required."
                 )
 
-    @property
-    def python26(self):
-        return skip_if(
-                lambda: sys.version_info < (2, 6),
-                "Python version 2.6 or greater is required"
-            )
-
-    @property
-    def python25(self):
-        return skip_if(
-                lambda: sys.version_info < (2, 5),
-                "Python version 2.5 or greater is required"
-            )
-
     @property
     def cpython(self):
         return only_if(lambda: util.cpython,
                "cPython interpreter needed"
              )
 
+
     @property
-    def bulletproof_pickle(self):
+    def non_broken_pickle(self):
         from sqlalchemy.util import pickle
         return only_if(
-            lambda: pickle.__name__ == 'cPickle' and \
-                sys.version_info < (3, 0) and not util.pypy,
-            "Needs Python 2.x cPickle"
+            lambda: not util.pypy and pickle.__name__ == 'cPickle'
+                or sys.version_info >= (3, 2),
+            "Needs cPickle+cPython or newer Python 3 pickle"
         )
 
+
     @property
     def predictable_gc(self):
         """target platform must remove all cycles unconditionally when
@@ -709,16 +694,11 @@ class DefaultRequirements(SuiteRequirements):
 
         """
         return skip_if(
-                lambda: (util.py3k or not util.py26) and
+                lambda: util.py3k and
                     self.config.options.enable_plugin_coverage,
-                "Stability issues with coverage + py3k, py2.5"
+                "Stability issues with coverage + py3k"
             )
 
-    @property
-    def english_locale_on_postgresql(self):
-        return skip_if(lambda: against('postgresql') \
-                    and not self.db.scalar('SHOW LC_COLLATE').startswith('en'))
-
     @property
     def selectone(self):
         """target driver must support the literal statement 'select 1'"""
